package com.gree.mall.manager.controller.material.manage;

import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.manager.annotation.ZfireList;
import com.gree.mall.manager.bean.material.manage.WebsitPartsSalesBean;
import com.gree.mall.manager.bean.material.manage.WebsitPartsSalesVO;
import com.gree.mall.manager.bean.material.manage.WebsitPartsSalesWVO;
import com.gree.mall.manager.enums.material.SourceSalesEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.helper.ResponseHelper;
import com.gree.mall.manager.logic.material.manage.WebsitPartsSalesOrderLogic;
import com.gree.mall.manager.plus.entity.WebsitStock;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Slf4j
@RestController
@Api(value = "网点配件销售单", tags = {"网点配件销售单"})
@RequestMapping(value = "/parts/sales/order", produces = "application/json; charset=utf-8")
public class WebsitPartsSalesOrderController {

    @Resource
    WebsitPartsSalesOrderLogic websitPartsSalesOrderLogic;

    @Resource
    RedisLockRegistry redisLockRegistry;

//    @Resource
//    WebsitSaleLogic websitSaleLogic;

    @ApiOperation("配件销售单新增")
    @PostMapping("/add")
    public ResponseHelper<WebsitPartsSalesBean> add(
            @RequestBody WebsitPartsSalesBean partsSalesOrderBean
    ) throws Exception {
        partsSalesOrderBean.setOrderSource(SourceSalesEnum.SELF.getKey());
        websitPartsSalesOrderLogic.add(partsSalesOrderBean);
        return ResponseHelper.success(partsSalesOrderBean);
    }

//    @ApiOperation("配件销售单修改")
//    @PostMapping("/update")
//    public ResponseHelper<PartsSalesOrderBean> update(
//            @RequestBody PartsSalesOrderBean partsSalesOrderBean
//    ) throws Exception {
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + partsSalesOrderBean.getSalesOrderNo());
//        if (!obtain.tryLock(5, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("配件销售单冻结中");
//        }
//        try {
//            websitPartsSalesOrderLogic.update(partsSalesOrderBean);
//        } finally {
//            obtain.unlock();
//        }
//
//        return ResponseHelper.success(partsSalesOrderBean);
//    }

    @ApiOperation("配件销售单根据id获取")
    @GetMapping("/loadSalesOrderById")
    public ResponseHelper<WebsitPartsSalesBean> loadSalesOrderById(
            @RequestParam String id
    ) throws RemoteServiceException {
        return ResponseHelper.success(websitPartsSalesOrderLogic.loadSalesOrderById(id, null));
    }

    @ZfireList
    @PostMapping("/list")
    @ApiOperation("配件销售列表")
    public ResponseHelper<IPage<WebsitPartsSalesVO>> list(
            @RequestBody ZfireParamBean zfireParamBean
    ) throws RemoteServiceException {
        IPage<WebsitPartsSalesVO> partsSalesOrderBeanIPage = websitPartsSalesOrderLogic.list(zfireParamBean);
        return ResponseHelper.success(partsSalesOrderBeanIPage, new TypeReference<WebsitPartsSalesVO>() {
        });
    }

    @ZfireList
    @PostMapping("/websit/list")
    @ApiOperation("网点配件销售列表")
    public ResponseHelper<IPage<WebsitPartsSalesWVO>> websitList(
            @RequestBody ZfireParamBean zfireParamBean
    ) throws RemoteServiceException {
        IPage<WebsitPartsSalesWVO> partsSalesOrderBeanIPage = websitPartsSalesOrderLogic.websitList(zfireParamBean);
        return ResponseHelper.success(partsSalesOrderBeanIPage, new TypeReference<WebsitPartsSalesWVO>() {
        });
    }

//    @PostMapping("/code")
//    @ApiOperation(value = "生成支付二维码")
//    public ResponseHelper<SalesPayList> generateCode(
//            @ApiParam(value = "网点销售单id", required = true) @RequestParam String sheetId,
//            @ApiParam(value = "网点支付配置id") @RequestParam(required = false) String payConfigId
//    ) throws RemoteServiceException, IOException, JDOMException {
//        return ResponseHelper.success(websitPartsSalesOrderLogic.generateCode(sheetId, payConfigId));
//    }

    @PostMapping("/list/export")
    @ApiOperation("配件销售列表导出")
    public void listExport(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        //2.查询要导出的内容
        IPage<WebsitPartsSalesVO> partsSalesOrderBeanIPage = websitPartsSalesOrderLogic.list(zfireParamBean);
        //3.导出
        FieldUtils.exportData(partsSalesOrderBeanIPage.getRecords(), zfireParamBean.getExportFields(), request, response);
    }

    @PostMapping("/websit/list/export")
    @ApiOperation("网点配件销售列表导出")
    public void shopListExport(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        //2.查询要导出的内容
        IPage<WebsitPartsSalesWVO> partsSalesOrderBeanIPage = websitPartsSalesOrderLogic.websitList(zfireParamBean);
        //3.导出
        FieldUtils.exportData(partsSalesOrderBeanIPage.getRecords(), zfireParamBean.getExportFields(), request, response);
    }

//    @ApiOperation("提交订单")
//    @GetMapping("/submit")
//    public ResponseHelper<PartsSalesOrderBean> submit(@ApiParam(value = "销售单id", required = true) @RequestParam String id) throws InterruptedException {
//        PartsSalesOrderBean partsSalesOrderBean = websitPartsSalesOrderLogic.loadSalesOrderById(id, null);
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + partsSalesOrderBean.getSalesOrderNo());
//        if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("销售单冻结中");
//        }
//        try {
//            return ResponseHelper.success(websitPartsSalesOrderLogic.submit(partsSalesOrderBean.getSalesOrderNo()));
//        } catch (Exception e) {
//            log.error("【提交订单处理】失败", e);
//            throw e;
//        } finally {
//            obtain.unlock();
//        }
//    }

//    @ApiOperation("现金支付")
//    @GetMapping("/change-cash")
//    public ResponseHelper<PartsSalesOrderBean> changeCash(@ApiParam(value = "销售单号", required = true) @RequestParam String salesOrderNo) throws Exception {
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + salesOrderNo);
//        if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("销售单冻结中");
//        }
//        try {
//            return ResponseHelper.success(websitPartsSalesOrderLogic.changeCash(salesOrderNo));
//        } catch (Exception e) {
//            log.error("【提交订单处理】失败", e);
//            throw e;
//        } finally {
//            obtain.unlock();
//        }
//    }
//
//    @ApiOperation("网点提货")
//    @GetMapping("/confirmDelivery")
//    public ResponseHelper shopConfirmDelivery(
//            @ApiParam(value = "销售单号", required = true) @RequestParam String salesOrderNo,
//            @ApiParam(value = "物流单号") @RequestParam(required = false) String logisticsNo
//    ) throws Exception {
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + salesOrderNo);
//        if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("销售单冻结中");
//        }
//        try {
//            websitPartsSalesOrderLogic.shopConfirmDelivery(salesOrderNo, logisticsNo);
//        } catch (Exception e) {
//            log.error("【网点提货处理】失败", e);
//            throw e;
//        } finally {
//            obtain.unlock();
//        }
//
//        return ResponseHelper.success();
//    }
//
//    @ApiOperation("删除")
//    @GetMapping("/deleteById")
//    public ResponseHelper deleteById(
//            @ApiParam(value = "销售单id", required = true) @RequestParam String id
//    ) throws RemoteServiceException {
//        websitPartsSalesOrderLogic.deleteById(id);
//        return ResponseHelper.success();
//    }
//
//    @ApiOperation("退回0-保存状态")
//    @GetMapping("/backToSave")
//    public ResponseHelper<PartsSalesOrderBean> backToSave(
//            @RequestParam String id
//    ) throws Exception {
//        PartsSalesOrderBean partsSalesOrderBean = websitPartsSalesOrderLogic.loadSalesOrderById(id, null);
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + partsSalesOrderBean.getSalesOrderNo());
//        if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("销售单冻结中");
//        }
//        try {
//            return ResponseHelper.success(websitPartsSalesOrderLogic.backToSave(partsSalesOrderBean.getSalesOrderNo()));
//        } catch (Exception e) {
//            log.error("【网点提货处理】失败", e);
//            throw e;
//        } finally {
//            obtain.unlock();
//        }
//
//    }
//
//    @ApiOperation("取消订单")
//    @GetMapping("/cancel")
//    public ResponseHelper cancel(@RequestParam String salesOrderNo) throws Exception {
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + salesOrderNo);
//        if (!obtain.tryLock(5, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("配件销售单冻结中");
//        }
//        try {
//            websitPartsSalesOrderLogic.cancel(salesOrderNo);
//        } catch (Exception e) {
//            log.error("【取消订单处理】失败", e);
//            throw e;
//        } finally {
//            obtain.unlock();
//        }
//
//        return ResponseHelper.success();
//    }

    @ApiOperation("获取配件可用库存")
    @GetMapping("/query/parts/new/qty")
    public ResponseHelper<WebsitStock> queryPartsNewQty(
            @ApiParam(required = true, value = "网点编码") @RequestParam String websitId,
            @ApiParam(required = true, value = "配件网点编号") @RequestParam String partsWebsitId,
            @ApiParam(required = true, value = "配件编码") @RequestParam String partsNumber
    ) {
        return ResponseHelper.success(websitPartsSalesOrderLogic.queryPartsNewQty(websitId, partsWebsitId, partsNumber));
    }

//    @ShopDataNotAuth
//    @GetMapping("/pay-config-list")
//    @ApiOperation(value = "网点支付配置列表")
//    public ResponseHelper<List<ShopPayConfig>> detail(
//            @ApiParam(value = "网点编号", required = true) @RequestParam String shopId,
//            @ApiParam(value = "类别", required = true) @RequestParam String category
//    ) throws RemoteServiceException {
//        List<ShopPayConfig> list = websitSaleLogic.payConfigList(shopId, category);
//        return ResponseHelper.success(list);
//    }
//
//    @ApiOperation("转换非申请")
//    @GetMapping("/trans-not-apply")
//    public ResponseHelper transNotApply(@RequestParam String salesOrderNo) throws Exception {
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.PARTS_SALES + salesOrderNo);
//        if (!obtain.tryLock(5, TimeUnit.SECONDS)) {
//            throw new RemoteServiceException("配件销售申请单冻结中");
//        }
//        try {
//            websitPartsSalesOrderLogic.transNotApply(salesOrderNo);
//        } catch (Exception e) {
//            log.error("【转换非申请处理】失败", e);
//            throw e;
//        } finally {
//            obtain.unlock();
//        }
//
//        return ResponseHelper.success();
//    }
}
